# -*- coding: utf-8 -*-
#
#  LanControl
#
#  $Id: plugin.py,v 0.1 2010/04/18 15:49:00 dre Exp $
#
#  Coded by dre (c) 2010
#  Coding idea and design by dre
#  Coding support by Dr.Best
#  Support: www.dreambox-tools.info
#
#  This plugin is licensed under the Creative Commons 
#  Attribution-NonCommercial-ShareAlike 3.0 Unported 
#  License. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
#
#  Alternatively, this plugin may be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially 
#  distributed other than under the conditions noted above.
#

from Components.ActionMap import *
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest
from Components.Pixmap import Pixmap
from Components.config import config, ConfigSubsection, ConfigSubList, ConfigNumber, ConfigText, configfile, ConfigInteger, ConfigIP, NoSave, getConfigListEntry, ConfigYesNo # Dr. Best --> ConfigInteger, ConfigIP
from Plugins.Plugin import PluginDescriptor
from Screens.HelpMenu import HelpableScreen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Directories import fileExists
from wol import WakeOnLan

from enigma import eListboxPythonMultiContent, gFont, RT_HALIGN_RIGHT, loadPNG

import os

# for localized messages
from . import _

config.plugins.lancontrol2 = ConfigSubsection()
config.plugins.lancontrol2.profile = ConfigSubList()
config.plugins.lancontrol2.autoquerystatus = ConfigYesNo(default = False)
config.plugins.lancontrol2.showinextensionsmenu = ConfigYesNo(default = False)
config.plugins.lancontrol2.profilecount = ConfigInteger(0) # Dr.Best --> Default Wert setzen, ausserdem auf ConfigInteger geaendert

def initProfileConfig():
	# hier baue ich eine neue CondigSubSection auf, und gebe die neue zurueck (siehe bei neu anlegen unten)
	s = ConfigSubsection()
	s.ipaddress =  ConfigText(default = "")
	s.macaddress = ConfigText(default = "")
	#s.status = NoSave(ConfigText(default = "yellow"))
	config.plugins.lancontrol2.profile.append(s)
	return s

def initConfig():
	count = config.plugins.lancontrol2.profilecount.value
	if count != 0:
		i = 0 # 0-basierender Index Aufpassen, wenn Du darauf zugreifen willst
		while i < count:
			initProfileConfig()
			i += 1

initConfig() # --> das ist wichtig, dass alle SubListItems initialisiert werden

##### Dr.Best Ende



class LanControlServerList(MenuList):
	def __init__(self, list):
		MenuList.__init__(self, list, False, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 18))
		self.l.setItemHeight(20)

def LanControlServerListEntry(ipaddress, macaddress, status="yellow"):
	res = [ (ipaddress, macaddress, status) ]
	png = "/usr/share/enigma2/skin_default/buttons/button_%s.png" %(status)
	if fileExists(png):
		res.append(MultiContentEntryPixmapAlphaTest(pos=(2, 2), size=(20, 20), png=loadPNG(png)))
	res.append(MultiContentEntryText(pos=(20, 0), size=(240, 20), font=0, text=ipaddress))
	res.append(MultiContentEntryText(pos=(260, 0), size=(240, 20), font=0, text=macaddress, flags=RT_HALIGN_RIGHT))
	return res


class LanControlOverviewScreen(Screen,HelpableScreen):
	skin= """
		<screen name="LanControlOverviewScreen" position="center,center" size="520,360" title="LanControl2 - Overview">
			<widget name="menulist" position="10,10" size="500,280" scrollbarMode="showOnDemand" />
			<widget name="ButtonRedtext" position="30,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonRed" pixmap="skin_default/buttons/button_red.png" position="10,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonGreentext" position="280,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="260,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonYellowtext" position="30,320" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="10,330" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonBluetext" position="280,320" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonBlue" pixmap="skin_default/buttons/button_blue.png" position="260,330" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""
	
	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close, _("Close plugin")),
			"ok":		(self.wakeUpServer, _("Start selected server")),
		}, -1)

		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":		(self.openPluginConfig, _("Open plugin config")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.removeServer,_("Remove selected server")),
			"green":	(self.addServer,_("Enter Add Server screen")),
			"yellow":	(self.queryStatus,_("Query server status")),
			"blue":		(self.wakeUpServer,_("Start selected server")),
		}, -1)			

		self["ButtonRed"] = Pixmap()
		self["ButtonRedtext"] = Label(_("Remove selected server"))
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Add server"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowtext"] = Label(_("Query server status"))
		self["ButtonBlue"] = Pixmap()
		self["ButtonBluetext"] = Label(_("Start selected server"))

		self.menulist = []
		self["menulist"] = LanControlServerList([])

		#menulist is built in getServers to use getServers for a refresh when returning to screen from addServer
		if config.plugins.lancontrol2.autoquerystatus.value == True:
			self.queryStatus()
		else:
			self.getServers()

	def addServer(self):
		self.session.openWithCallback(self.getServers, addServerScreen, None)

	def getServers(self):
		self.menulist = []
		count = config.plugins.lancontrol2.profilecount.value
		if count != 0:
			for i in range(0,count):
				ipaddress = config.plugins.lancontrol2.profile[i].ipaddress.value
				macaddress = config.plugins.lancontrol2.profile[i].macaddress.value
				self.menulist.append(LanControlServerListEntry(ipaddress,macaddress))
			self["ButtonRed"].show()
			self["ButtonRedtext"].show()
			self["ButtonYellow"].show()
			self["ButtonYellowtext"].show()
			self["ButtonBlue"].show()
			self["ButtonBluetext"].show()
		else:
			self["ButtonRed"].hide()
			self["ButtonRedtext"].hide()
			self["ButtonYellow"].hide()
			self["ButtonYellowtext"].hide()
			self["ButtonBlue"].hide()
			self["ButtonBluetext"].hide()
		self["menulist"].l.setList(self.menulist)

	def removeServer(self):
		sel = self["menulist"].getSelectedIndex()
		config.plugins.lancontrol2.profilecount.value = config.plugins.lancontrol2.profilecount.value - 1
		config.plugins.lancontrol2.profilecount.save()
		del config.plugins.lancontrol2.profile[sel]
		config.plugins.lancontrol2.profile.save()
		config.plugins.lancontrol2.save()
		configfile.save()
		self.getServers()

	def queryStatus(self):
		self.menulist = []
		count = config.plugins.lancontrol2.profilecount.value
		if count != 0:
			for i in range(0, count):
				ipaddress = config.plugins.lancontrol2.profile[i].ipaddress.value
				macaddress = config.plugins.lancontrol2.profile[i].macaddress.value
				self.pingresult = os.system("ping -c 1 -q %s" % (config.plugins.lancontrol2.profile[i].ipaddress.value))
				if (int(self.pingresult)/256)==0:
					#config.plugins.lancontrol2.profile[i].status.value = "green"
					status = "green"
				else:
					#config.plugins.lancontrol2.profile[i].status.value = "red"
					status = "red"
				#self.menulist.append(LanControlServerListEntry(ipaddress,macaddress,config.plugins.lancontrol2.profile[i].status.value))
				self.menulist.append(LanControlServerListEntry(ipaddress,macaddress,status))
			self["ButtonRed"].show()
			self["ButtonRedtext"].show()
			self["ButtonYellow"].show()
			self["ButtonYellowtext"].show()
			self["ButtonBlue"].show()
			self["ButtonBluetext"].show()
		else:
			self["ButtonRed"].hide()
			self["ButtonRedtext"].hide()
			self["ButtonYellow"].hide()
			self["ButtonYellowtext"].hide()
			self["ButtonBlue"].hide()
			self["ButtonBluetext"].hide()
		self["menulist"].l.setList(self.menulist)				

	def wakeUpServer(self):
		index = self["menulist"].getSelectedIndex()
		#mac-addresse fuer aktuell selektierten server aus der config lesen und an WakeOnLan uebergeben
		server = config.plugins.lancontrol2.profile[index].macaddress.value
		WakeOnLan(server)
		info = self.session.open(MessageBox,_("The command to boot the server has been sent"), MessageBox.TYPE_INFO, timeout=5)
		info.setTitle(_("LanControl2"))

	def openPluginConfig(self):
		self.session.open(PluginConfigScreen)

class addServerScreen(Screen,HelpableScreen):
	skin="""
		<screen name="addServerScreen" position="center,center" size="520,320" title="LanControl2 - Add Server">
			<widget name="menulist" position="10,10" size="500,280" scrollbarMode="showOnDemand" />
			<widget name="ButtonGreentext" position="30,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
			<widget name="ButtonYellowtext" position="280,280" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonYellow" pixmap="skin_default/buttons/button_yellow.png" position="260,290" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close,_("Return to Overview screen")),
		}, -1)			

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveDevice,_("Save selected server")),
			"yellow":	(self.openSearchScreen,_("Open Search screen")),
		}, -1)			

		self.menulist = []
		self.serverlist = []
		self["menulist"] = LanControlServerList([])
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save selected server"))
		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowtext"] = Label(_("Search by IP address"))

		#menulist is built in readInterfaces to use readInterfaces for a refresh when returning to screen from searchMACAddress
		self.readInterfaces()

	def readInterfaces(self):
		self.menulist = []
		self.serverlist = []
		interfaces = {}

		try:
			fp = file('/proc/net/arp', 'r')
			interfaces = fp.readlines()
			fp.close()
		except:
			print "[LanControl2] arp - opening failed"
		
		for i in interfaces:
			split = i.strip().split(' ')	
			
			if len(split[0])==15:
				self.macaddress = "%s" % split[20]
			elif len(split[0])==14:
				self.macaddress = "%s" % split[21]
			elif len(split[0])==13:
				self.macaddress = "%s" % split[22]
			elif len(split[0])==12:
				self.macaddress = "%s" % split[23]
			elif len(split[0])==11:
				self.macaddress = "%s" % split[24]
			elif len(split[0])==10:
				self.macaddress = "%s" % split[25]
			elif len(split[0])==9:
				self.macaddress = "%s" % split[26]
			elif len(split[0])==8:
				self.macaddress = "%s" % split[27]
			elif len(split[0])==7:
				self.macaddress = "%s" % split[28]
			else:
				continue

			self.serverlist.append([split[0],self.macaddress])
			self.menulist.append(LanControlServerListEntry(split[0],self.macaddress,"green"))
			if len(self.serverlist) == 0:
				self["ButtonGreen"].hide()
				self["ButtonGreentext"].hide()
			else:
				self["ButtonGreen"].show()
				self["ButtonGreentext"].show()
		self["menulist"].l.setList(self.menulist)

	def saveDevice(self):
		# Dr.Best
		current = initProfileConfig() # neue ConfigSubsection in Object current
		index = self["menulist"].getSelectedIndex() # Dein Index
		# speicher die Werte in der ConfigSubsection
		current.ipaddress.value = self.serverlist[index][0]
		current.macaddress.value = self.serverlist[index][1]
		# speichern
		current.save()
		
		# erhoehe den Zaehler von profilecount um 1 und speichern
		config.plugins.lancontrol2.profilecount.value +=1
		config.plugins.lancontrol2.profilecount.save()
		# speicher die Config fuer das lancontrol2
		config.plugins.lancontrol2.save()
		# speicher configfile		
		configfile.save()

		info = self.session.open(MessageBox,_("Selected entry has been saved"), MessageBox.TYPE_INFO, timeout=5)
		info.setTitle(_("LanControl2"))

	def openSearchScreen(self):
		self.session.openWithCallback(self.readInterfaces, searchMACAddressScreen)

class searchMACAddressScreen(Screen,ConfigListScreen,HelpableScreen):
	skin="""
		<screen name="addServerScreen" position="center,center" size="520,60" title="LanControl2 - Search MACAddress">
			<widget name="config" position="10,10" size="500,30" scrollbarMode="showOnDemand"/>
			<widget name="ButtonGreentext" position="30,30" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,40" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.searchMACAddress,_("Search MACAddress")),
		}, -1)

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close,_("Return to AddServer screen")),
		}, -1)

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Search"))

		self.list = []
		self.ipbox = NoSave(ConfigIP(default = [0,0,0,0]))
		self.list.append(getConfigListEntry(_("Search MAC address with IP address:"), self.ipbox))

		ConfigListScreen.__init__(self, self.list)
		self["config"].setList(self.list)
	
	def searchMACAddress(self):
		self.pingresult = os.system("ping -c 1 -q %d.%d.%d.%d" % tuple(self.ipbox.value))
		if (int(self.pingresult)/256)==0:
			self.close()
		else:
			error = self.session.open(MessageBox,_("Couldn't ping IP address. Is the server running?"), MessageBox.TYPE_ERROR, timeout=5)
			error.setTitle(_("LanControl2"))

class PluginConfigScreen(Screen,ConfigListScreen,HelpableScreen):
	skin="""
		<screen name="PluginoConfigScreen" position="center,center" size="520,110" title="LanControl2 - Plugin Config">
			<widget name="config" position="10,10" size="500,60" scrollbarMode="showOnDemand"/>
			<widget name="ButtonGreentext" position="30,70" size="230,40" valign="center" halign="left" zPosition="10" font="Regular;18" transparent="1" />
			<widget name="ButtonGreen" pixmap="skin_default/buttons/button_green.png" position="10,80" zPosition="10" size="16,16" transparent="1" alphatest="on" />
		</screen>"""

	def __init__(self,session,args=0):
		Screen.__init__(self,session)
		HelpableScreen.__init__(self)
		self.session = session

		self["WizardActions"]  = HelpableActionMap(self, "WizardActions",
		{
			"back":		(self.close,_("Return to Overview screen")),
		}, -1)

		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.saveandclose,_("Save config and return to Overview screen")),
		}, -1)

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreentext"] = Label(_("Save and Close"))

		self.list = []
		self.list.append(getConfigListEntry(_("Query status on plugin start"), config.plugins.lancontrol2.autoquerystatus))
		self.list.append(getConfigListEntry(_("Show plugin in extensions menu"), config.plugins.lancontrol2.showinextensionsmenu))

		ConfigListScreen.__init__(self, self.list)
		self["config"].setList(self.list)

	def saveandclose(self):
		for x in self.list:
			x[1].save()
		self.close()

	def cancel(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close()

def main(session, **kwargs):
	session.open(LanControlOverviewScreen)

def Plugins(**kwargs):
	list = [PluginDescriptor(name="LanControl",
		description="Wake-On-LAN",where = [PluginDescriptor.WHERE_PLUGINMENU], icon="lancontrol2.png",fnc=main)]
	if config.plugins.lancontrol2.showinextensionsmenu.value:
		list.append(PluginDescriptor(name="LanControl", description="Wake-On-LAN", where = [PluginDescriptor.WHERE_EXTENSIONSMENU], fnc=main))
	return list